'
' Scrolling text demo for a Sure 2416 24*16 Dot Matrix Display Board
' Written by Les Johnson for the Proton BASIC Compiler
'
    Include "Amicus18.inc"                  ' Configure the compiler for a PIC18F25K20 at 64MHz. i.e. An Amicus18 board
    Include "2416.inc"                      ' Load the Sure 2416 display interface routines into the program
'
' Declare variables
'
    Dim HT_bIndex As Byte System            ' Pointer within the string to display
    Dim HT_bFontLine As Byte System         ' Position within the font for the line of pixels

    Dim HT_bScroll_StartX As Byte           ' Holds the X position For the start of the scrolling area
    Dim HT_bScroll_EndX As Byte             ' Holds the X position For the End of the scrolling area
    Dim HT_bScroll_StartY As Byte           ' Holds the Y position of the Line To scroll

    GoTo Main
'-------------------------------------------------------------------------------------
' Scroll Left, n lines on the display
'
' Input     : HT_bScroll_StartY holds the Y position of the line to scroll
'             HT_bScroll_StartX holds the X position for the start of the scrolling area
'             HT_bScroll_EndX holds the X position for the end of the scrolling area
' Output    : None
' Notes     : None
'
Scroll_Left:
    HT_bCol = HT_bScroll_EndX
    Repeat
        HT_bRow = 7
        Repeat
            HT_Pixel(HT_bCol + 1,HT_bScroll_StartY + HT_bRow, tPixelColor) ' Read a pixel
            HT_Plot(HT_bCol,HT_bScroll_StartY + HT_bRow)   ' Place the pixel one position left
            Dec HT_bRow
        Until HT_bRow = 0
        Inc HT_bCol
    Until HT_bCol > HT_bScroll_StartX
    Return
'-------------------------------------------------------------------------------------
' Scroll a message across the display
'
' Input     : Cdata table "Message_Text", holds the text to display (null terminated)
' Output    : None
' Notes     : Re-uses a lot of variables declared in the "2416.inc" file
ScrollMessage:
    HT_tPixelColSave = tPixelColor              ' Save the tPixelColor value
    TABLAT = CRead8 Message_Text[HT_bIndex]     ' Get a character from the string
    '
    ' Display a line of pixels from the character
    '
    HT_wCharOffset = TABLAT * 6                 ' 6 horizontal bytes per font
    HT_wCharOffset = HT_wCharOffset + HT_bFontLine ' Add the offset to the character's value
    TABLAT = CRead8 ScFont[HT_wCharOffset]        ' Read the byte from the font table
'
' Send the value in TABLAT to the Display as a line of vertical pixels
'
    HT_bRow = 7                                 ' 7 vertical pixels per font byte
    Repeat
        tPixelColor = TABLAT.6
        HT_Plot(HT_bScroll_StartX,HT_bScroll_StartY + HT_bRow)
        TABLAT = TABLAT << 1
        Dec HT_bRow
    Until HT_bRow = 0
    Inc HT_bFontLine                            ' Move to the next line of pixels in the character
    GoSub Scroll_Left                           ' Scroll the line of text left by one pixel
    If HT_bFontLine >= 6 Then                   ' Have we done 6 pixels?
        HT_bFontLine = 0                        ' Yes. So reset the counter
        Inc HT_bIndex                           ' Move to the next character in the string
        If HT_bIndex >= Len(Message_Text) Then  ' Have we reached the end of the string?
            HT_bIndex = 0                       ' Yes. So start at the beginning
        EndIf
    EndIf
    tPixelColor = HT_tPixelColSave              ' Restore the tPixelColor value
    Return
'-------------------------------------------------------------------------------------
'
' The text to scroll on the display
' Must be null terminated
' MUst be less than or equal to 255 characters in length
'
Message_Text:
    CData "Hello World. This is a scrolling message using the Amicus18 board and the Sure 2416 display. ", 0
'
'-------------------------------------------------------------------------------------
' The main program starts here
'
Main:
    HT_bIndex = 0                               ' Reset the scrolling tex string pointer
    HT_bFontLine = 0                            ' Reset the text font line counter

    HT_bScroll_StartY = 3                       ' Set Y position to 3
    HT_bScroll_StartX = 23                      ' Start scrolling at pixel 23
    HT_bScroll_EndX = 0                         ' End the scroll at position 0


    Do                                          ' Create an infinite loop
        GoSub ScrollMessage                     ' Scroll the message
        DelayMS 50                              ' Cause a delay to see what's going on
    Loop                                        ' Do it forever
'
' Load the font used by characters
'
    Include "ScrollFont.inc"
